
/*
Un programme qui montre comment faire des fades-in, fades-out avec de multiples objets
A multiple object fade-in/fade-out program
Copyright (C) 2007 BEYLER Jean Christophe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


#include "Image.h"
#include <SDL_image.h>

/* 
 * FADE_TIME : temps que prend les animations FADE_IN et FADE_OUT pour
 * se termine. En millisecondes...
 */ 
#define FADE_TIME 3500

/* Fonction statique */
static void Image_RandomMove(SImage *im); /* Bouge l'image d'un delta aleatoirement */

/* Fonction de base */
SImage *Image_Create(char *fname)
{
    /* Allocation memoire */
    SImage *res = malloc(sizeof(*res));
    SDL_Surface *tmp;

    /* Verification */
    if(res==NULL){
        return NULL;
    }

    /* Chargement de l'image */
    tmp = SDL_LoadBMP(fname);
    /* Verification */
    if(res->im == NULL) {
        free(res);
        return NULL;
    }

    /* 
     * Appel a DisplayFormat : cette image aura le meme format que celle
     * qui gere l'ecran
     */ 
    res->im = SDL_DisplayFormat(tmp);
    /* Liberation de la surface tmp */
    SDL_FreeSurface(tmp);

    /* Verification */
    if(res->im == NULL) {
        free(res);
        return NULL;
    }

    /* Valeur par defaut et heure actuelle */
    res->r.x = 0;
    res->r.y = 0;
    res->alpha = 0;
    res->last = SDL_GetTicks();

    /* Retour de la structure */
    return res;
}

/* Destruction d'une image */
void Image_Destroy(SImage *im)
{
    /* Si l'image n'est pas NULL */
    if(im != NULL) {
        SDL_FreeSurface(im->im), im->im = NULL;
        free(im);
    }
}

/* On change la valeur alpha */
void Image_ChangeAlpha(SImage *im, unsigned int alpha)
{
    if(im != NULL) {
        /* Verification */
        if(alpha>255) {
            alpha = 255;
        }

        im->alpha = alpha;

        SDL_SetAlpha(im->im, SDL_SRCALPHA, alpha);
    }
}

/* Positionnement aleatoire */
void Image_RandomPos(SImage *im, SDL_Rect *r)
{
    double d;
    if((im != NULL)&&(r != NULL)) {
        if(im->im != NULL) {
            /* On teste la largeur et la hauteur */
            if((im->im->w < r->w ) && (im->im->h < r->h)) {
                d = rand();         /* On choisit un nombre aleatoire  */
                d /= RAND_MAX;      /* 0.0 < d <= 1.0 */
                d *= r->w-im->im->w;/* 0.0 < d <= r->w - im->im->w*/
                d += r->x;          /* r->x < d < (r->x + im->im->w) */
                im->r.x = d;

                d = rand();             /* On choisit un nombre aleatoire */
                d /= RAND_MAX;          /* 0.0 < d <= 1.0 */
                d *= r->h-im->im->h;    /* 0.0 < d <= r.y - im->im->h */
                d += r->y;              /* r.y < d <= (r->y + im->im->h) */
                im->r.y = d;
            }
        }
    }
}

/* Animation functions */
/* Fade in initialisation */
void Image_FadeIn_Init(SImage* im)
{
    if(im != NULL) {
        /* Valeur par defaut 0 pour que la surface soit transparente */
        Image_ChangeAlpha(im, 0);
        /* Met a jour le dernier changement */
        im->last = SDL_GetTicks();
    }
}

/* Fade in : test de fin */
int Image_FadeIn_IsFinished(SImage* im)
{
    if(im == NULL) {
        return 1;
    }

    return im->alpha >= SDL_ALPHA_OPAQUE;
}

/* Fade in : fonction de rendu */
void Image_FadeIn_Frame(SImage* im, SDL_Surface *screen)
{
    unsigned int diff, interp;
    if(im != NULL) {
        /* Calcul de l'interpolation de la valeur alpha */
        diff = SDL_GetTicks() - im->last;
        diff = (diff>FADE_TIME)?FADE_TIME:diff;
        interp = (int) ((255.0 * diff)/FADE_TIME);

        /* On change la valeur alpha */
        Image_ChangeAlpha(im, interp);

        /* On copie l'image */
        SDL_BlitSurface(im->im, NULL, screen, &(im->r));
    }
}

/* Fade out : initialisation */
void Image_FadeOut_Init(SImage* im)
{
    if(im != NULL) {
        /* Valeur de depart alpha : l'image sera opaque */
        Image_ChangeAlpha(im, 255);
        /* Met a jour le dernier changement */
        im->last = SDL_GetTicks();
    }
}

/* Fade in : test de fin */
int Image_FadeOut_IsFinished(SImage* im)
{
    if(im == NULL) {
        return 1;
    }

    return im->alpha <= 0;
}

/* Fade in : fonction de rendu */
void Image_FadeOut_Frame(SImage* im, SDL_Surface *screen)
{
    unsigned int diff, interp;
    if(im != NULL) {
        /* Calcul de l'interpolation de la valeur alpha */
        diff = SDL_GetTicks() - im->last;
        diff = (diff>FADE_TIME)?FADE_TIME:diff;
        interp = (int) ((255.0 * (FADE_TIME - diff))/FADE_TIME);

        /* On change la valeur alpha */
        Image_ChangeAlpha(im, interp);

        /* On copie l'image */
        SDL_BlitSurface(im->im, NULL, screen, &(im->r));

    }
}

/* Mouvement aleatoire */
void Image_RandomMove(SImage *im)
{
    if(im != NULL) {
        int oldx = im->r.x;
        int oldy = im->r.y;

        oldx += (rand()%3)-1;
        oldy += (rand()%3)-1;

        im->r.x = oldx;
        im->r.y = oldy;
    }
}

/* Fade out move : fonction de rendu */
void Image_FadeOutMove_Frame(SImage *im, SDL_Surface *screen)
{
    Image_FadeOut_Frame(im, screen);
    Image_RandomMove(im);
}

/* Fade in move : fonction de rendu */
void Image_FadeInMove_Frame(SImage *im, SDL_Surface *screen)
{
    Image_FadeIn_Frame(im, screen);
    Image_RandomMove(im);
}
